/* untag.c, HTML untagging

 * Copyright (C) 1998 J.A. Bezemer
 * Changed, 2001/08/18, M.C. Bezemer, removed global variables
 *
 * Licensed under the terms of the GNU General Public License.
 * ABSOLUTELY NO WARRANTY.
 * See the file `COPYING' in this directory.
 */


#include "untag.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


void
add_to_tagstring (char *addition, unsigned long l /*= length*/,
		  char **tagstring, unsigned long *tagstringlen,
		  unsigned long *tagstringpos )
{
  /* *tagstringpos points to "final" \0 char in string, that will be
     overwritten by first char of addition => pos == string length */
  if (*tagstringpos + l + 1 > *tagstringlen)
    {
      *tagstringlen = *tagstringpos + l + 2000;
      *tagstring = realloc (*tagstring, *tagstringlen);
    }

  memcpy (*tagstring + *tagstringpos, addition, l);
  *tagstringpos += l;
  (*tagstring)[*tagstringpos] = 0;
}


char *
untag (char *s)
/* Renders all non-alnum chars in s to &#num; entities. Returns pointer
   to string that is valid only until next call of _this_ function
   (i.e. uses different buffer than l() ).
   (i.e. DON'T USE TWICE WITHIN ONE PRINTF!!) */
{
  static char *tagstring = NULL;
  unsigned long tagstringlen = 0;
  unsigned long tagstringpos = 0;

  register unsigned long slen = 0;
  register unsigned long curpos = 0;
  unsigned long startpos = 0;
  register unsigned char c;
  char entity[20];

  if (s==NULL) {
    return NULL;
  }
  slen=strlen (s);
  free (tagstring);
  tagstring = NULL;
  tagstringlen = 0;
  tagstringpos = 0;

  while (curpos < slen)
    {
      c = s[curpos];

      while (((c >= '?' && c <= '~') ||		/* incl. A-Z, a-z */
	      (c >= ' ' && c <= '!') ||
	      (c >= '\'' && c <= ';') ||	/* incl. 0-9 */
	      (c >= '#' && c <= '%') ||
	      c == '=' ||
	      c == '\n')
	     && curpos < slen)
	{
	  curpos++;
	  c = s[curpos];
	}

      if (curpos > startpos)
	add_to_tagstring (s + startpos, curpos - startpos,
			  &tagstring, &tagstringlen, &tagstringpos);

      if (curpos < slen)
	{
	  snprintf (entity, 19, "&#%d;", c);
	  add_to_tagstring (entity, strlen (entity),
			    &tagstring, &tagstringlen, &tagstringpos);
	}
      curpos++;
      startpos = curpos;
    }

  /* make sure string does exists and is terminated correctly */
  tagstring = realloc (tagstring, tagstringpos + 1);
  tagstring[tagstringpos] = 0;

  return tagstring;
}
