/* urlencode.c, HTML URL encoding (like PHP's rawurlencode()-function)

 * Copyright (C) 2001,2002 M.C. Bezemer
 *
 * Licensed under the terms of the GNU General Public License.
 * ABSOLUTELY NO WARRANTY.
 * See the file `COPYING' in this directory.
 */


#include "urlencode.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


void add_to_urlstring(char *addition,
		      unsigned long l /*= length*/,
		      char **urlstring,
		      unsigned long *urlstringlen,
		      unsigned long *urlstringpos)
{
	/* *urlstringpos points to "final" \0 char in string, that will be
	   overwritten by first char of addition => pos == string length */
	if (*urlstringpos + l + 1 > *urlstringlen) {
		*urlstringlen = *urlstringpos + l + 2000;
		*urlstring = realloc(*urlstring, *urlstringlen);
	}

	memcpy(*urlstring + *urlstringpos, addition, l);
	*urlstringpos += l;
	(*urlstring)[*urlstringpos] = 0;
}


char *urlencode(char *s)
/* Renders all non-alnum chars in s to %xx entities. Returns pointer
   to string that is valid only until next call of _this_ function
   (i.e. uses different buffer than l() ).
   (i.e. DON'T USE TWICE WITHIN ONE PRINTF!!) */
{
	static char *urlstring = NULL;
	unsigned long urlstringlen = 0;
	unsigned long urlstringpos = 0;

	register unsigned long slen = 0;
	register unsigned long curpos = 0;
	unsigned long startpos = 0;
	register unsigned char c;
	char entity[20];

	if (s==NULL) {
		return NULL;
	}
	slen=strlen(s);
	free(urlstring);
	urlstring = NULL;
	urlstringpos = 0;
	urlstringlen = 0;

	while (curpos < slen) {
		c = s[curpos];

		while (((c >= 'A' && c <= 'Z') ||
			(c >= 'a' && c <= 'z') ||
			(c >= '0' && c <= '9') ||
			(c == '-') ||
			(c == '_') ||
			(c == '.'))
		       && curpos < slen) {
			curpos++;
			c = s[curpos];
		}

		if (curpos > startpos)
			add_to_urlstring(s + startpos,
					 curpos - startpos,
					 &urlstring,
					 &urlstringlen,
					 &urlstringpos);

		if (curpos < slen) {
			snprintf(entity, 19, "%%%.2X", c);
			add_to_urlstring(entity,
					 strlen(entity),
					 &urlstring,
					 &urlstringlen,
					 &urlstringpos);
		}
		curpos++;
		startpos = curpos;
	}

	/* make sure string does exists and is terminated correctly */
	urlstring = realloc(urlstring, urlstringpos + 1);
	urlstring[urlstringpos] = 0;

	return urlstring;
}
